"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = build;
var suffixToRemove = "Container";

function addDisplayNameStatement(t, varName) {
  var pos = varName.indexOf(suffixToRemove);
  var displayName = 0 < pos ? varName.substring(0, pos) : varName; // <VAR>.displayName = '<VAR>';

  return t.expressionStatement( //
  t.assignmentExpression("=", //
  t.memberExpression(t.identifier(varName), t.identifier("displayName")), // <VAR>.displayName
  t.stringLiteral(displayName)));
}

function extractModuleName(fileOpts) {
  var sourceFileName = fileOpts.sourceFileName;
  var name = sourceFileName && sourceFileName.split("\\").pop().split("/").pop(); // for perfs

  var dotPos = name && name.indexOf(".");
  return dotPos > 0 ? name.substring(0, dotPos) : name || "Unknown";
}

function build(_ref) {
  var t = _ref.types;
  return {
    visitor: {
      /*
       Rewrite
         Reselect.memoize?(x, y, fn)
       To
         Reselect.memoize("<module>:<functionName>", [x, y], fn)
       */
      CallExpression: function CallExpression(path, state) {
        if (t.isMemberExpression(path.node.callee)) {
          var callee = path.node.callee;

          if (t.isIdentifier(callee.property) && callee.property.name.startsWith("memoize")) {
            if (t.isIdentifier(callee.object) && callee.object.name === "Reselect") {
              if (path.node.arguments.length > 1 && !t.isStringLiteral(path.node.arguments[0])) {
                // We found a call expression : Reselect.memoize??(x, y, ...)
                var parentFunctionNode = path.getFunctionParent().node;

                if (parentFunctionNode) {
                  var name = t.isIdentifier(parentFunctionNode.id) ? parentFunctionNode.id.name : "anonymous";
                  var extraParams = []; // if parent function, add params as extra inputs

                  var grandParentFunction = path.getFunctionParent().getFunctionParent();

                  if (grandParentFunction) {
                    if (grandParentFunction.node.params && grandParentFunction.node.params.length > 0) {
                      // var x = function(y) { ... }
                      extraParams = grandParentFunction.node.params;

                      if (t.isIdentifier(grandParentFunction.node.id)) {
                        name = grandParentFunction.node.id.name;
                      } else {
                        if (t.isVariableDeclarator(grandParentFunction.parent)) {
                          name = grandParentFunction.parent.id.name;
                        }
                      }
                    }
                  } // function getSomething(...) { Reselect.memoize() =


                  var moduleName = extractModuleName(state.file.opts); // anonymous with grandParent

                  var functionName = name;
                  var inputs = [].concat(path.node.arguments); // create new array

                  var computeFunction = inputs.pop();
                  var newInputs = inputs.concat(extraParams);
                  var newArgs = [t.stringLiteral(moduleName + ':' + functionName), t.arrayExpression(newInputs), computeFunction];
                  path.replaceWith(t.callExpression( //
                  t.memberExpression(t.identifier('Reselect'), t.identifier('memoize')), newArgs));
                }
              }
            }
          }
        }
      },
      VariableDeclaration: function VariableDeclaration(path, state) {
        if (1 === path.node.declarations.length) {
          var declarator = path.node.declarations[0];

          if (t.isIdentifier(declarator.id)) {
            var varName = declarator.id.name;
            var initExpression = declarator.init;
            var _state$opts$methodNam = state.opts.methodNames,
                methodNames = _state$opts$methodNam === void 0 ? ["reactStamp"] : _state$opts$methodNam;

            if (methodNames && t.isCallExpression(initExpression)) {
              var callee = initExpression.callee;

              if (t.isMemberExpression(initExpression.callee)) {
                callee = callee.object.callee;
              }

              if (callee && methodNames.indexOf(callee.name) >= 0) {
                var statementPath = t.isExportNamedDeclaration(path.parentPath) ? path.parentPath : path;
                statementPath.insertAfter(addDisplayNameStatement(t, varName));
              }
            }
          }
        }
      }
    }
  };
}